const express = require("express");
const compression = require("compression");
const app = express();
const Docker = require("dockerode");
const docker = new Docker();
const ws = require("express-ws")(app);
const cors = require("cors");
const { checkSchema } = require("express-validator");
const { convert } = require("./docker/names");
app.use(cors());
app.use(compression());
app.use(express.json());

app.get("/", (_, res) => {
  res.send("Merhaba Dünya!");
});

app.get("/containers", (_, res) => {
  docker.listContainers({ all: true }, function (_, containers) {
    res.send(
      containers
        .filter(
          (c) =>
            c.Image.startsWith("nyxteam/oyunparki-small") ||
            c.Image.startsWith("nyxteam/oyunparki-full")
        )
        .map((c) => {
          c.Names[0] = convert(c.Names[0]);
          return c;
        })
    );
  });
});

app.post(
  "/images",
  checkSchema({
    image: {
      in: ["nyxteam/oyunparki-full", "nyxteam/oyunparki-small"],
      errorMessage: "Yanlış versiyon seçildi!",
    },
  }),
  (req, res) => {
    docker.listImages(function (_, images) {
      const status =
        images.filter((i) => i.RepoTags[0].startsWith(req.body.image))
          .length !== 0;
      res.status(status ? 200 : 422).end();
    });
  }
);

app.post(
  "/create",
  checkSchema({
    image: {
      in: ["nyxteam/oyunparki-full", "nyxteam/oyunparki-small"],
      errorMessage: "Yanlış versiyon seçildi!",
    },
    username: {
      nullable: true,
    },
    password: {
      nullable: true,
    },
    max_ram: { isInt: true, toInt: true },
    spice_password: {
      isBoolean: true,
      toBoolean: true,
    },
  }),
  (req, res) => {
    require("./docker/controllers/create")(req, res);
  }
);

app.post("/restart", (req, res) => {
  res.send(docker.getContainer(req.body.container).restart());
});

app.post("/stop", (req, res) => {
  const container = docker.getContainer(req.body.container);
  container
    .stop()
    .then(() => {
      res.status(200).end();
    })
    .catch(() => {
      res.status(500).end();
    });
});

app.post("/remove", (req, res) => {
  const container = docker.getContainer(req.body.container);
  container
    .remove({
      force: true,
    })
    .then(() => {
      res.status(200).end();
    })
    .catch(() => {
      res.status(500).end();
    });
});

require("./socket")(app);

app.listen(30000);

module.exports = app;
